import styles from "./DosHeader.module.scss";
import { Flex } from "antd";
import React, { useContext, useEffect, useRef, useState } from "react";
import type { GetRef, InputRef, TableProps } from "antd";
import { Form, Input, Table } from "antd";

type FormInstance<T> = GetRef<typeof Form<T>>;

const EditableContext = React.createContext<FormInstance<any> | null>(null);

interface Item {
  name: string; // 字段名称
  offset: string; // 偏移
  size: Number; // 字段大小
  value: string; // 字段值
  children?: Item[];
}

interface EditableRowProps {
  index: number;
}

const EditableRow: React.FC<EditableRowProps> = ({ index, ...props }) => {
  const [form] = Form.useForm();
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  );
};

interface EditableCellProps {
  title: React.ReactNode;
  editable: boolean;
  dataIndex: keyof Item;
  record: Item;
  handleSave: (record: Item) => void;
}

const EditableCell: React.FC<React.PropsWithChildren<EditableCellProps>> = ({
  title,
  editable,
  children,
  dataIndex,
  record,
  handleSave,
  ...restProps
}) => {
  const [editing, setEditing] = useState(false);
  const inputRef = useRef<InputRef>(null);
  const form = useContext(EditableContext)!;

  useEffect(() => {
    if (editing) {
      inputRef.current?.focus();
    }
  }, [editing]);

  const toggleEdit = () => {
    setEditing(!editing);
    form.setFieldsValue({ [dataIndex]: record[dataIndex] });
  };

  const save = async () => {
    try {
      const values = await form.validateFields();
      toggleEdit();
      handleSave({ ...record, ...values });
    } catch (errInfo) {
      console.log("Save failed:", errInfo);
    }
  };

  let childNode = children;

  if (editable) {
    childNode = editing ? (
      <Form.Item
        style={{ margin: 0 }}
        name={dataIndex}
        rules={[{ required: true, message: `${title} is required.` }]}
      >
        <Input size="small" ref={inputRef} onPressEnter={save} onBlur={save} />
      </Form.Item>
    ) : (
      <div
        className="editable-cell-value-wrap"
        style={{ paddingInlineEnd: 24 }}
        onClick={toggleEdit}
      >
        {children}
      </div>
    );
  }

  return <td {...restProps}>{childNode}</td>;
};

interface DataType {
  name: string; // 字段名称
  offset: string; // 偏移
  size: string | Number; // 字段大小
  value: string; // 字段值
  field_type: string; // 字段类型
  children?: DataType[];
}

type ColumnTypes = Exclude<TableProps<DataType>["columns"], undefined>;

const App = ({ defaultData = [] }) => {
  const [dataSource, setDataSource] = useState<DataType[]>(defaultData);
  useEffect(()=>{
    setDataSource(defaultData)
  }, [defaultData])

  const defaultColumns: (ColumnTypes[number] & {
    editable?: boolean;
    dataIndex: string;
  })[] = [
    {
      title: "字段名称",
      dataIndex: "name",
      width: "30%",
    },
    {
      title: "偏移量",
      dataIndex: "offset",
    },
    {
      title: "字段类型",
      dataIndex: "field_type",
    },
    {
      title: "字段大小",
      dataIndex: "size",
    },
    {
      title: "字段值",
      dataIndex: "value",
      editable: true,
    },
  ];

  const replaceNodeByName = (
    data: DataType[],
    name: string,
    newNode: DataType
  ): DataType[] => {
    return data.map((item) => {
      if (item.name === name) {
        return newNode;
      }
      if (item.children) {
        return {
          ...item,
          children: replaceNodeByName(item.children, name, newNode),
        };
      }
      return item;
    });
  };

  const handleSave = (row: DataType) => {
    let newData = [...dataSource];
    newData = replaceNodeByName(newData, row.name, row);
    setDataSource(newData);
  };

  const components = {
    body: {
      row: EditableRow,
      cell: EditableCell,
    },
  };

  const columns = defaultColumns.map((col) => {
    if (!col.editable) {
      return col;
    }
    return {
      ...col,
      onCell: (record: DataType) => ({
        record,
        editable: col.editable,
        dataIndex: col.dataIndex,
        title: col.title,
        handleSave,
      }),
    };
  });

  return (
    <Table<DataType>
      components={components}
      rowClassName={() => "editable-row"}
      bordered
      rowKey={(record) => record.name}
      style={{ width: "100%" }}
      dataSource={dataSource}
      columns={columns as ColumnTypes}
      pagination={false}
      size="small"
    />
  );
};

export default function DosHeader({ defaultData }) {
  console.log("DosHeader defaultData:", defaultData);
  return (
    <Flex className={styles.root}>
      <App defaultData={defaultData} />
    </Flex>
  );
}
